from typing import List

from langchain_core.retrievers import BaseRetriever
from langchain_core.callbacks import CallbackManagerForRetrieverRun
from langchain_core.documents import Document

from tada_ai.client import TadaAIClient
from tada_ai.models.chunk_options import ChunkOptions
from tada_ai.models.reranker_options import RerankerOptions


class TadaAiRetriever(BaseRetriever):
    api_key: str | None = None
    chunks: ChunkOptions | None = None
    reranker: RerankerOptions | bool = True
    space_id: str | None = None

    def _get_relevant_documents(
        self, query: str, *, run_manager: CallbackManagerForRetrieverRun
    ) -> List[Document]:
        client = TadaAIClient(api_key=self.api_key)
        result = client.search(
            prompt=query,
            space_id=self.space_id,
            reranker=self.reranker,
            chunks=self.chunks,
        )

        return [Document(page_content=chunk.markdown) for chunk in result.chunks]



# coding: utf-8

"""
    Tada AI API

    API for access Tada resources

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

from pydantic import Field, StrictStr
from typing import Optional
from tada_ai.models.api_model import ApiModel
from tada_ai.models.chunk_options import ChunkOptions
from tada_ai.models.reranker_options import RerankerOptions
from typing import Optional


class SearchQueryInput(ApiModel):
    prompt: StrictStr
    space_id: Optional[StrictStr] = Field(default=None)
    chunk_options: Optional[ChunkOptions] = Field(default=None)
    reranker: Optional[RerankerOptions] = Field(default=None)



from __future__ import annotations

from pydantic import Field, StrictFloat, StrictInt
from typing import Optional, Union
from typing import Optional

from tada_ai.models.api_model import ApiModel


class ChunkOptions(ApiModel):
    requested_min_word_count: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=50,
        description="Will attempt to keep the minimum chunk size to less than the provided amount. NOTE: This is not guaranteed. Please validate if your system requires a minimum or maximum size.",
    )
    requested_max_word_count: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=1500,
        description="Will attempt to keep the maximum chunk size to less than the provided amount. NOTE: This is not guaranteed. Please validate if your system requires a minimum or maximum size.",
    )



from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class ApiModel(BaseModel):
    model_config = model_config = ConfigDict(
        alias_generator=to_camel, populate_by_name=True
    )



from __future__ import annotations

from pydantic import Field, StrictBool, StrictFloat, StrictInt
from typing import Optional
from typing import Optional

from tada_ai.models.api_model import ApiModel


class RerankerOptions(ApiModel):
    use_reranker: Optional[StrictBool] = Field(
        default=True, description="Turns the reranker on or off."
    )
    top_n: Optional[StrictInt] = Field(
        default=10,
        description="The largest number of chunks to return from re-ranking",
    )
    threshold: Optional[StrictFloat] = Field(
        default=0.25, description="The minimum threshold for a chunk"
    )



# coding: utf-8

# flake8: noqa

"""
    Tada AI API

    API for access Tada resources
"""  # noqa: E501


__version__ = "0.3.1"

from tada_ai.client import TadaAIClient


# import ApiClient
from tada_ai.exceptions import ApiError
from tada_ai.exceptions import ApiTypeError
from tada_ai.exceptions import ApiValueError
from tada_ai.exceptions import ApiKeyError
from tada_ai.exceptions import ApiAttributeError
from tada_ai.exceptions import ApiError

# import models into sdk package
from tada_ai.models.chunk_options import ChunkOptions
from tada_ai.models.search_query_input import SearchQueryInput
from tada_ai.models.reranker_options import (
    RerankerOptions,
)
from tada_ai.models.search_record_chunk import SearchRecordChunk
from tada_ai.models.search_result import SearchResult
from tada_ai.models.space import Space
from tada_ai.models.space_file import SpaceFile
from tada_ai.models.space_files_list import SpaceFilesList
from tada_ai.models.space_list import SpaceList



# coding: utf-8

# flake8: noqa
"""
    Tada AI API

    API for access Tada resources

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from tada_ai.models.create_manual_space_input import CreateManualSpaceInput
from tada_ai.models.chunk_options import ChunkOptions
from tada_ai.models.search_query_input import SearchQueryInput
from tada_ai.models.reranker_options import RerankerOptions
from tada_ai.models.search_record_chunk import SearchRecordChunk
from tada_ai.models.search_result import SearchResult
from tada_ai.models.space import Space
from tada_ai.models.space_file import SpaceFile
from tada_ai.models.space_files_list import SpaceFilesList
from tada_ai.models.space_list import SpaceList



from typing import Literal, Union
from pydantic import Field

from tada_ai.models.api_model import ApiModel

import csv
from io import StringIO


class Location(ApiModel):
    x0: float
    y0: float
    x1: float
    y1: float
    page_num: int


class BaseContentBlock(ApiModel):
    id: str
    full_word_count: int


class TableCellContentBlock(BaseContentBlock):
    type: Literal["TableCell"] = "TableCell"
    location: Location
    text: str
    rows: tuple[int, ...]
    cols: tuple[int, ...]

    def dfs(self):
        yield self


class TableContentBlock(BaseContentBlock):
    type: Literal["Table"] = "Table"
    locations: list[Location]
    children: list[TableCellContentBlock]

    def to_csv(self):
        col_row_value: dict[tuple[int, int], str] = dict()
        for cell in self.children:
            for col in cell.cols:
                for row in cell.rows:
                    col_row_value[(col, row)] = cell.text

        # Find the dimensions of the table
        max_row = max(max(cell.rows) for cell in self.children)
        max_col = max(max(cell.cols) for cell in self.children)

        csv_file = StringIO()
        writer = csv.writer(csv_file, quoting=csv.QUOTE_MINIMAL, lineterminator="\n")

        for row_num in range(max_row + 1):
            row = [
                col_row_value.get((col_num, row_num), "")
                for col_num in range(max_col + 1)
            ]
            writer.writerow(row)

        csv_file.seek(0)
        return csv_file.read()

    def to_markdown(self, first_line_as_header=False) -> str:
        col_row_value: dict[tuple[int, int], str] = dict()
        for cell in self.children:
            for col in cell.cols:
                for row in cell.rows:
                    col_row_value[(col, row)] = cell.text.replace("|", r"\|").replace(
                        "\n", "<br>"
                    )

        max_row = max(max(cell.rows) for cell in self.children)
        max_col = max(max(cell.cols) for cell in self.children)

        rows = []
        start = 0
        if first_line_as_header:
            row = " | ".join(
                col_row_value.get((col_num, 0), "") for col_num in range(max_col + 1)
            )
            rows.append(f"| {row} |")
            row = " | ".join(["-------"] * (max_col + 1))
            rows.append(f"| {row} |")
            start = 1

        for row_num in range(start, max_row + 1):
            row = " | ".join(
                col_row_value.get((col_num, row_num), "")
                for col_num in range(max_col + 1)
            )
            rows.append(f"| {row} |")

        return "\n".join(rows)

    def dfs(self):
        yield self
        yield from self.children


class RootContentBlock(BaseContentBlock):
    type: Literal["Root"] = "Root"
    children: list[
        Union[
            "HeaderContentBlock",
            "ListContentBlock",
            "ParagraphContentBlock",
            "TableContentBlock",
        ]
    ] = Field(discriminator="type")

    def __iter__(self):
        yield from self.dfs()

    def dfs(self):
        """
        A Pre-order depth-first-search implementation.
        This returns all content blocks in the order that they appear in the document
        """
        yield self
        for c in self.children:
            yield from c.dfs()


class HeaderContentBlock(BaseContentBlock):
    type: Literal["Header"] = "Header"
    locations: list[Location]
    text: str
    children: list[
        Union[
            "HeaderContentBlock",
            "ListContentBlock",
            "ParagraphContentBlock",
            "TableContentBlock",
        ]
    ] = Field(discriminator="type")

    def dfs(self):
        yield self
        for c in self.children:
            yield from c.dfs()


class ParagraphContentBlock(BaseContentBlock):
    type: Literal["Paragraph"] = "Paragraph"
    locations: list[Location]
    text: str

    def dfs(self):
        yield self


class ListContentBlock(BaseContentBlock):
    type: Literal["List"] = "List"
    locations: list[Location]
    children: list[Union["ListItemContentBlock", "ListContentBlock"]] = Field(
        discriminator="type"
    )

    def dfs(self):
        yield self
        for c in self.children:
            yield from c.dfs()


class ListItemContentBlock(BaseContentBlock):
    type: Literal["ListItem"] = "ListItem"
    locations: list[Location]
    text: str

    def dfs(self):
        yield self


ContentBlock = (
    HeaderContentBlock
    | ListContentBlock
    | ListItemContentBlock
    | ParagraphContentBlock
    | TableContentBlock
    | TableCellContentBlock
)



import os
from typing import Optional
import typing
import requests
from tada_ai.exceptions import ApiException
from tada_ai.models.chunk_options import ChunkOptions
from tada_ai.models.content_blocks import RootContentBlock
from tada_ai.models.search_query_input import SearchQueryInput
from tada_ai.models.reranker_options import (
    RerankerOptions,
)
from tada_ai.models.search_result import SearchResult
from tada_ai.models.space import Space
from tada_ai.models.space_file import SpaceFile
from tada_ai.models.space_files_list import SpaceFilesList
from tada_ai.models.space_list import SpaceList

DEFAULT_BASE_URL = "https://api.tadatoday.ai/api"

import io


class BaseService:
    def __init__(self, api_key: str, *, base_url: str):
        self.api_key = api_key
        self._base_url = base_url

    def _url(self, *path: str):
        return _join_url(self._base_url, *path)


class TadaAIClient(BaseService):
    def __init__(
        self, api_key: str | None = None, *, base_url: str = DEFAULT_BASE_URL
    ) -> None:
        api_key = (
            os.environ.get("TADA_AI_API_KEY", None) if api_key is None else api_key
        )
        if api_key is None:
            raise Exception(
                '"TADA_AI_API_KEY" not found in env. Either set the environment variable, or pass in the key manually'
            )

        return super().__init__(api_key, base_url=base_url)

    def search(
        self,
        prompt: str,
        *,
        space_id: str | None = None,
        reranker: RerankerOptions | bool = True,
        chunks: Optional[ChunkOptions] = None,
    ):
        url = self._url("search")

        if reranker == False:
            rerankerOptions = None
        elif reranker == True:
            rerankerOptions = RerankerOptions(use_reranker=True)
        else:
            rerankerOptions = reranker

        input = SearchQueryInput(
            prompt=prompt,
            space_id=space_id,
            reranker=rerankerOptions,
            chunk_options=chunks,
        )
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"ApiKey {self.api_key}",
        }
        response = requests.post(
            url, json=input.model_dump(by_alias=True), headers=headers
        )
        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return SearchResult.model_validate_json(response.text)

    @property
    def spaces(self):
        return SpaceService(self.api_key, base_url=self._base_url)

    @property
    def space_files(self):
        return SpaceFileService(self.api_key, base_url=self._base_url)


class SpaceFileService(BaseService):
    def get(self, space_file_id: str):
        url = self._url("space-files", space_file_id)
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"ApiKey {self.api_key}",
        }
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return SpaceFile.model_validate_json(response.text)

    def list(self, space_id: str):
        url = self._url("spaces", space_id, "files")
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"ApiKey {self.api_key}",
        }
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return SpaceFilesList.model_validate_json(response.text)

    def create(self, space_id: str, file: typing.BinaryIO, path: str, mime_type: str):
        url = self._url("spaces", space_id, "files")
        headers = {"Authorization": f"ApiKey {self.api_key}"}
        files = {"file": (path, file, mime_type)}
        try:
            response = requests.post(url, headers=headers, files=files)
        except UnicodeDecodeError:
            raise Exception(
                "The file contains bytes, but was opened as text. Did you mean to do `open('my-file', 'rb')`?"
            )

        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return SpaceFile.model_validate_json(response.text)

    def content_blocks(self, space_file_id: str):
        url = self._url("space-files", space_file_id, "content-blocks")
        headers = {"Authorization": f"ApiKey {self.api_key}"}
        response = requests.get(url, headers=headers)

        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return RootContentBlock.model_validate_json(response.text)


class SpaceService(BaseService):
    def create(self, name: str):
        url = self._url("spaces")
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"ApiKey {self.api_key}",
        }
        response = requests.post(url, json={"name": name}, headers=headers)
        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return Space.model_validate_json(response.text)

    def get(self, space_id: str):
        url = self._url("spaces", space_id)
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"ApiKey {self.api_key}",
        }
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return Space.model_validate_json(response.text)

    def list(self):
        url = self._url("spaces")
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"ApiKey {self.api_key}",
        }
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise ApiException.from_response(
                http_resp=response, body=response.text, data=None
            )

        return SpaceList.model_validate_json(response.text)


def _join_url(base_url: str, *path_items: str):
    if base_url.endswith("/"):
        base_url = base_url[:-1]
    path = [
        path_item[1:] if path_item.startswith("/") else path_item
        for path_item in path_items
    ]

    path.insert(0, base_url)
    return "/".join(path)



# coding: utf-8

"""
    Tada AI API

    API for access Tada resources

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

from tada_ai.models.api_model import ApiModel


class SearchRecordChunk(ApiModel):
    id: str
    markdown: str = Field(description="The markdown representation of the chunk")
    score: Optional[float] = Field(
        description="The score returned by the reranker. Between 0 and 1"
    )
